/*____________________________________________________________________________*\
 *

 Copyright (c) 1997 John Roy. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    (http://www.johnroy.com/pi3/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: Pi3Win.c$
 * $Date: Sun Aug 10 06:40:13 1997$
 *
 Description:

\*____________________________________________________________________________*/
/* $SourceTop:$ */

#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <assert.h>

#include "DblList.h"
#include "PIString.h"

#include "resrc1.h"
#include "Dialogs.h"
#include "Config.h"

/*____________________________________________________________________________*\
 *
 Description:
	Definitions and global values
\*____________________________________________________________________________*/
static HWND hWndList = 0;
static HWND hWndType = 0;
static HWND hWndPath = 0;
static HWND hWndAdd = 0;
static HWND hWndReplace = 0;
static HWND hWndDelete = 0;
static DblList lTypes;
static DblList lPaths;

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#if 0
static const char *__pTypes[]=
{ 
"Access (CLF)",
"Referer Logfile",
0
};
#endif

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static void Internal_addLine( const char *pLine )
{
	assert( pLine );
	assert( lTypes.Size()==lPaths.Size() );

	int i;
	/* --- scan to tab --- */
	for( i=0; pLine[i] && pLine[i]!='\t'; i++ );
	
	lTypes.Append( (DblList::type)PI_NEW( PIString( pLine, i ) ) );

	/* --- scan over whitespace --- */
	for( ; pLine[i] && isspace(pLine[i]); i++ );
	lPaths.Append( (DblList::type)PI_NEW( PIString( &(pLine[i]) ) ) );	

	assert( lTypes.Size()==lPaths.Size() );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static void Internal_updateButtons( WPARAM wParam, LPARAM lParam )
{
	int iCBSel = SendMessage( hWndType, CB_GETCURSEL, 0, 0 );
	const char *pType = 0;

	int iExistingIndex = SendMessage( hWndList, LB_GETCURSEL, 0, 0 );

	/*
	** Set buttons
	*/
	if ( iCBSel<0 || SendMessage( hWndType, CB_GETLBTEXTLEN, iCBSel, 0 )==0 )
		{
		if ( IsWindowEnabled( hWndAdd ) ) EnableWindow( hWndAdd, FALSE );
		if ( IsWindowEnabled( hWndReplace ) ) EnableWindow( hWndReplace, FALSE );
		}
	else if ( SendMessage( hWndPath, EM_LINELENGTH, 0, 0 )!=0 )
		{
		if ( iExistingIndex!=LB_ERR )/* --- disable add, enable replace --- */
			{
			if ( IsWindowEnabled( hWndAdd ) ) EnableWindow( hWndAdd, FALSE );
			if ( !IsWindowEnabled( hWndReplace ) ) EnableWindow( hWndReplace, TRUE );
			}
		else						/* --- disable replace, enable add --- */
			{
			if ( !IsWindowEnabled( hWndAdd ) ) EnableWindow( hWndAdd, TRUE );
			if ( IsWindowEnabled( hWndReplace ) ) EnableWindow( hWndReplace, FALSE );
			};
		};

	/*
	** If there is a selected item in the listbox then
	** enable delete, otherwise disable delete	
	*/
	if ( SendMessage( hWndList, LB_GETCURSEL, 0, 0 )==LB_ERR )
		{ EnableWindow( hWndDelete, FALSE ); }
	else
		{ EnableWindow( hWndDelete, TRUE ); };
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
static BOOL CALLBACK DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam,
	LPARAM lParam)
{
	enum { BUF_SIZE=1023 };
	char szBuf[BUF_SIZE+1];
	int aTabstop[2];

	switch (uMsg)
		{
		case WM_INITDIALOG:
			AD_hourglassOn();
			{
			const char *pTmp;
			int iTmp;
			hWndList = GetDlgItem( hDlg, IDC_LIST );
			hWndType = GetDlgItem( hDlg, IDC_TYPE );
			hWndPath= GetDlgItem( hDlg, IDC_PATH );
			hWndAdd = GetDlgItem( hDlg, IDC_ADD );
			hWndReplace = GetDlgItem( hDlg, IDC_REPLACE );
			hWndDelete= GetDlgItem( hDlg, IDC_DELETE );

			assert( hWndList && hWndType && hWndPath );
			assert( hWndAdd && hWndReplace && hWndDelete );

			EnableWindow( hWndAdd, FALSE );
			EnableWindow( hWndReplace, FALSE );
			EnableWindow( hWndDelete, FALSE );

			/*
			** Load all logging types.
			*/
			iTmp = 0;
			for(;; iTmp++ )
				{
				pTmp = AD_lookupValueEx( __pConfig, "Internal", 
					"LoggingTypes", iTmp );
				if ( !pTmp )
					{ break; };

				::SendMessage( hWndType, CB_ADDSTRING, 0, (LPARAM)pTmp );
				};

			/*
			** Limit edit field lengths, really just a sanity thing
			*/
			SendMessage( hWndPath, EM_SETLIMITTEXT, 256, 0 );

			/*
			** Setup tabstops
			*/
			aTabstop[0] = 0;
			aTabstop[1] = 75;
			SendMessage( hWndList, LB_SETTABSTOPS, 2, (LPARAM)&aTabstop );

			/*
			** Load all mime types
			*/
			iTmp = 0;
			for(;; iTmp++ )
				{
				pTmp = AD_lookupValueEx( __pConfig, "Logfiles", 
					"Logging", iTmp );
				if ( !pTmp )
					{ break; };

				if ( SendMessage( hWndList, LB_ADDSTRING, 0, (LPARAM)pTmp )>=0 )
					{
					Internal_addLine( pTmp );
					}
				else
					{ break; /* error */; };
				};
			}

			AD_hourglassOff();
			break;

		case WM_DESTROY:
			{
			int i;
			int iCount = SendMessage( hWndList, LB_GETCOUNT, 0, 0 );

			/* --- remove old ones first --- */
			DblListIterator x( lTypes );
			DblListIterator y( lPaths );
			for( i=0; i<iCount; i++, x++, y++ )
				{
				/*
				** Delete this entry in lists
				*/
				assert( !x.BadIndex() && !y.BadIndex() );
				PI_DELETE( (PIString *)x.Current() );
				PI_DELETE( (PIString *)y.Current() );
				};
			lTypes.Clear();
			lPaths.Clear();
			};
			break;

		case WM_COMMAND:
			switch (GET_WM_COMMAND_ID(wParam, lParam))
				{
				case IDC_ADD:
					{
					int i;
					*szBuf = '\0';
					SendMessage( hWndType, WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
					for( i=0; i<BUF_SIZE && szBuf[i] && szBuf[i]!='\t'; i++ );
					if ( szBuf[i] ) { szBuf[i] = '\0'; };
					lTypes.Append( (DblList::type)PI_NEW( PIString( szBuf ) ) );
					szBuf[i++] = '\t';
					SendMessage( hWndPath, WM_GETTEXT, BUF_SIZE-i, (LPARAM)&(szBuf[i]) );
					lPaths.Append( (DblList::type)PI_NEW( PIString( &(szBuf[i]) ) ) );
					SendMessage( hWndList, LB_ADDSTRING, 0, (LPARAM)szBuf );

					/*
					** Clear edit fields, focus to list
					*/
					SendMessage( hWndType, CB_SETCURSEL, (WPARAM)-1, 0 );
					SendMessage( hWndPath, WM_SETTEXT, 0, (LPARAM)"" );
					SetFocus( hWndList );
					PropSheet_Changed( GetParent( hDlg ), hDlg );
					};
					break;
	
				case IDC_REPLACE:
					{
					/*
					** Get selected item index 
					*/
					int iCurSel = SendMessage( hWndList, LB_GETCURSEL, 0, 0 );

					/*
					** Get text to add and add it to internal lists only
					*/
					int i;
					*szBuf = '\0';
					SendMessage( hWndType, WM_GETTEXT, BUF_SIZE, (LPARAM)szBuf );
					for( i=0; i<BUF_SIZE && szBuf[i] && szBuf[i]!='\t'; i++ );
					if ( szBuf[i] ) { szBuf[i] = '\0'; };
					lTypes.Append( (DblList::type)PI_NEW( PIString( szBuf ) ) );
					szBuf[i++] = '\t';
					SendMessage( hWndPath, WM_GETTEXT, BUF_SIZE-i, (LPARAM)&(szBuf[i]) );
					lPaths.Append( (DblList::type)PI_NEW( PIString( &(szBuf[i]) ) ) );
	
					/*
					** Delete the item from the list and internal lists
					*/
					SendMessage( hWndList, LB_DELETESTRING,	iCurSel, 0 );
					PI_DELETE( (PIString *)lTypes.Detach( iCurSel) );
					PI_DELETE( (PIString *)lPaths.Detach( iCurSel) );
					
					/*
					** Add new item to listbox
					*/
					SendMessage( hWndList, LB_ADDSTRING, 0, (LPARAM)szBuf );

					/*
					** Clear edit fields, focus to list
					*/
					SendMessage( hWndType, CB_SETCURSEL, (WPARAM)-1, 0 );
					SendMessage( hWndPath, WM_SETTEXT, 0, (LPARAM)"" );
					SetFocus( hWndList );
					PropSheet_Changed( GetParent( hDlg ), hDlg );
					};
					break;

				case IDC_DELETE:
					{
					int iCurSel = SendMessage( hWndList, LB_GETCURSEL, 0, 0 );
					SendMessage( hWndList, LB_DELETESTRING,	iCurSel, 0 );
					PI_DELETE( (PIString *)lTypes.Detach( iCurSel) );
					PI_DELETE( (PIString *)lPaths.Detach( iCurSel) );
			
					/*
					** Clear edit fields, focus to list
					*/
					SendMessage( hWndType, CB_SETCURSEL, (WPARAM)-1, 0 );
					SendMessage( hWndPath, WM_SETTEXT, 0, (LPARAM)"" );
					SetFocus( hWndList );
					PropSheet_Changed( GetParent( hDlg ), hDlg );
					};
					break;

				case IDC_TYPE:
					if ( GET_WM_COMMAND_CMD(wParam, lParam)==CBN_SELCHANGE )
						{ 
						SendMessage( hWndList, LB_SETCURSEL, (WPARAM)-1, 0 );
						Internal_updateButtons( wParam, lParam ); 
						};
					break;

				case IDC_PATH:
					if ( GET_WM_COMMAND_CMD(wParam, lParam)==EN_CHANGE )
						{ Internal_updateButtons( wParam, lParam ); };
					break;

				case IDC_LIST:
					if ( GET_WM_COMMAND_CMD(wParam, lParam)==LBN_SELCHANGE )
						{
						int iSel = SendMessage( hWndList, LB_GETCURSEL, 0, 0 );
						if ( iSel!=LB_ERR )
							{
							assert( iSel>=0 && iSel<lTypes.Size() );
							EnableWindow( hWndDelete, TRUE );
							int iIndex = SendMessage( hWndType, CB_FINDSTRINGEXACT, 
								(WPARAM)-1, 
								(LPARAM)(const char *)*((PIString *)lTypes[iSel]) );
							SendMessage( hWndType, CB_SETCURSEL, (WPARAM)
								(iIndex==CB_ERR ? -1 : iIndex), 0 );
							SendMessage( hWndPath, WM_SETTEXT, 0,
								(LPARAM)(const char *)*((PIString *)lPaths[iSel]) );
							};
						};
					break;

				default:;		
				}
			break;

		case WM_NOTIFY:
			{
			switch( ((NMHDR *)lParam)->code )
				{
				case PSN_QUERYCANCEL:
					return AD_close( hDlg );

				case PSN_KILLACTIVE:
					{
					int i;
					int iCount = SendMessage( hWndList, LB_GETCOUNT, 0, 0 );

					/*
					** Save all mime types
					*/
					/* --- remove old ones first --- */
					AD_deleteValue( __pConfig, "Logfiles", "Logging" );
					for( i=0; i<iCount; i++ )
						{
						*szBuf = '\0';
						szBuf[BUF_SIZE] = '\0';
						/*
						** safety only
						*/
						if ( SendMessage( hWndList, LB_GETTEXTLEN, 0, 0 ) >= BUF_SIZE )
							{
							MessageBox( hDlg, "Logging specification too long", 
								"Internal Error", MB_OK );
							break;
							};
						SendMessage( hWndList, LB_GETTEXT, i, (LPARAM)szBuf );
						AD_addValue( __pConfig, "Logfiles", "Logging", szBuf );
						};
					};
					SetWindowLong( hDlg, DWL_MSGRESULT, FALSE );
					break;

				default:;
				};
			};
			break;

		default:
				return (FALSE);
		};

	return (TRUE);
}

/*____________________________________________________________________________*\
 *
 Description:
	Definitions and global values
\*____________________________________________________________________________*/
DLGPROC fnLogging = (DLGPROC)DialogProc;
