/*____________________________________________________________________________*\
 *

 Copyright (c) 1997 John Roy. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    (http://www.johnroy.com/pi3/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: Action.cpp$
 * $Date: Sun Aug 10 06:36:43 1997$
 *
 Description:
	Perform generic action.

\*____________________________________________________________________________*/
//$SourceTop:$

#include "Pi2API.h"
#include "Pi3Expr.h"
#include "PICompat.h"
#include "HandBase.h"
#include "PIStrStr.h"

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#define KEY_CONF_PI3EXPRESSION		"Pi3Expression"

/*____________________________________________________________________________*\
 *
 Description:
	Documentation.
\*____________________________________________________________________________*/
#if 0
	/*
	** HTML documentation for this handler
	*/
/*___+++HTMLDOC_BEGIN+++___*/
Name:
	Action

Description:
	Evaluate a Pi3Expression to cause a side effect.

Options:
<H5>Overview</H5>
<TABLE BORDER=1>
<TH>Option
<TH>Default
<TH>Values
<TH>Short Description
<TH>Example(s)

<TR>
<TD>Pi3Expression
<TD>+
<TD>A Pi3Expression
<TD>Pi3Expression to evaulate
<TD>Pi3Expression="&dbreplace(response,string,ObjectMap,Scripts)"

</TABLE>
<STRONG>-</STRONG> in the <IT>default</IT> indicates no default<BR>
<STRONG>+</STRONG> in the <IT>default</IT> indicates the field is mandatory<BR>

<H4>Description of Options</H4>
<H5>
	Pi3Expression
</H5>
Specifies the Pi3Expression that will be evaulated. 

Phase:
	Any

Returns:
	PIAPI_ERROR if this handler was invoked for an inappropriate phase, 
	otherwise PIAPI_CONTINUE is returned.

Example:
	<PRE>
	&ltObject&gt
		Name Action
		Class ActionClass
	&lt/Object&gt

	&ltObject&gt
		...
		CheckType Action Pi3Expression="&dbreplace(response,rfc822,\
			Content-Type,text/plain)"
		...
	&lt/Object&gt

	</PRE>
/*___+++HTMLDOC_END+++___*/
#endif

/*____________________________________________________________________________*\
 *
 Class:
 Description:
\*____________________________________________________________________________*/
class Action : public HandlerBaseHTTP
{
private:
	/* ---
	Configuration data
	--- */
	Pi3Expression *pExpr;			/* expression to execute */

	/* --- parameters --- */
	int Parameter( const char *pVariable, const char *pValue,
		const char *pWhere )
		{
		assert( pVariable );
		assert( pValue );
		assert( pWhere );

		PIOStrStream os;
		os << pWhere << "SendPi3Expression: ";

		if ( !PIUtil_stricmp( KEY_CONF_PI3EXPRESSION, pVariable ) )
			{
			if ( pExpr )
				{
				os << "Pi3Expession already defined." << ends;
				CONFIG_ERR( Object(), os.str() );
				return 0;
				};
			Pi3String *pError = Pi3String_new( 0 );
			pExpr = Pi3Expression_new( pValue, 0, pError );
			if ( !pExpr )
				{
				os << Pi3String_getPtr( pError ) << ends;
				Pi3String_delete( pError );
				CONFIG_ERR( Object(), os.str() );
				delete pExpr;
				pExpr = 0;
				return 0;
				};
			Pi3String_delete( pError );
			}
		else
			{
			os << "Unknown directive '" << pVariable <<
				"'" << ends;
			CONFIG_ERR( Object(), os.str() );
			return 0;
			};

		return 1; 	/* --- not implemented --- */
		};

public:
	Action( PIObject *pTheObject, int iArgc, const char *ppArgv[] )
	:
		HandlerBaseHTTP( pTheObject ),
		pExpr( 0 )
		{
		ReadParameters( iArgc, ppArgv );

		/* --- 
		check values are valid
		--- */
		if ( !pExpr )
			{
			CONFIG_ERR( Object(), "Action: 'Pi3Expression' \
not defined or error in definition." );
			SetOK( 0 );
			return;
			};
		};

	~Action()
		{
		Pi3Expression_delete( pExpr );
		};

	inline int Handle( int /* iPhase */, PIHTTP &tPIHTTP, PIIOBuffer &/* tB */ )
		{
		if ( !Pi3Expression_write( pExpr, &tPIHTTP, 0, 0, 0 ) )
			{
			return PIAPI_CONTINUE;
			};

		return PIAPI_CONTINUE;
		};
};

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int Action_constructor( PIObject *pObj,
	int iArgc, const char *ppArgv[] )
{
	return HandlerBaseHTTP_constructor( pObj, PI_NEW( Action( pObj,
		iArgc, ppArgv ) ) );
}

#if 0
/*___+++CNF_BEGIN+++___*/
	<Class>
		Name ActionClass
		Type LogicExtension
		Library HTTP
		OnClassLoad HandlerBaseHTTP_onClassLoad
		Constructor Action_constructor
		CopyConstructor HandlerBaseHTTP_copyConstructor
		Destructor HandlerBaseHTTP_destructor
		Execute HandlerBaseHTTP_execute
	</Class>

	<Object>
		Name Action
		Class ActionClass
	</Object>

/*___+++CNF_END+++___*/
#endif

