/*____________________________________________________________________________*\
 *

 Copyright (c) 1997 John Roy. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    (http://www.johnroy.com/pi3/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: HandBase.cpp$
 * $Date: Sun Aug 10 06:37:24 1997$
 *
 Description:

\*____________________________________________________________________________*/
//$SourceTop:$

#include <assert.h>
#include <ctype.h>

#include "IFaceBse.h"
#include "PIString.h"
#include "DeQuote.h"

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
IFaceBase::IFaceBase( PIObject *pTheObject )
:
	pObject( pTheObject ),
	iOK( 1 ),
    pInterface( 0 )
{
    PIObject_setDB( pObject, PIDB_new(
        PIObject_getDB( pObject ), 
        PIObject_getName( pObject ) ) );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
void IFaceBase::ReadParameters( int iArgc, const char *ppArgv[] )
{
	if ( !PIObject_readParameters( pObject, iArgc, ppArgv, ParameterFn,
		this ) )
		{
		iOK = 0;
		}
	else
		{
		iOK = 1;
		};
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
IFaceBase::~IFaceBase()
{
    PIDB *pDB = PIObject_getDB( pObject );
    PIObject_setDB( pObject, PIDB_getParent( pDB ) );
    PIDB_delete( pDB );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int IFaceBase::Execute( int iArgc, const char *ppArgv[] )
{
	PI_MARK;

	if ( iArgc!=1 )
		{ return PIAPI_ERROR; };	

	Interface *pTheInterface = (Interface *)ppArgv[0];	
	if ( !pTheInterface )
		{ return PIAPI_ABORT; };

    if ( pTheInterface->iMessage==INTRFACE_MSG_INIT )
        {
        pInterface = pTheInterface;
        };

	int iRet = PIAPI_ABORT;
    iRet = Handle( *pTheInterface );

	PI_MARK;
	return iRet;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int IFaceBase::Close( HWND )
{
#if 0
    ::DestroyWindow( __hPropSheet );
    ::ExitProcess( 0 ):
#endif
    return (TRUE);
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int IFaceBase_execute( PIObject *pObj, int iArgc,
		const char *ppArgv[] )
{
	if ( !pObj ) return PIAPI_ERROR;

	return ((IFaceBase *)PIObject_getUserData(pObj))->
		Execute( iArgc, ppArgv );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int IFaceBase_onClassLoad( PIClass_LoadAction eLoad, int,
	const char *[] )
{
	switch( eLoad )
		{
		case STARTUP:
		default:;
		};
	return PIAPI_COMPLETED;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int IFaceBase_constructor( PIObject *pObj, IFaceBase *pHandler )
{
	if (!( pHandler && pHandler->IsOK() ))
		{
		delete pHandler;
		return PIAPI_ERROR;
		};
	if ( PIObject_setUserData( pObj, pHandler ) )
		{
		delete pHandler;
		CONFIG_ERR( pObj, "IFaceBase_constructor: PIObject_setUserData() \
failed" );
		return PIAPI_ERROR;
		};
	return PIAPI_COMPLETED;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int IFaceBase_copyConstructor( PIObject *, int,
    const char *[] )
{
	/* --- forbid copy constructor --- */
	return PIAPI_ERROR;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int IFaceBase_destructor( PIObject *pObj, int,
	const char *[] )
{
	delete (IFaceBase *)PIObject_getUserData( pObj );
	return PIAPI_COMPLETED;
}

