/*____________________________________________________________________________*\
 *

 Copyright (c) 1997 John Roy. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    (http://www.johnroy.com/pi3/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: _PIClass.cpp$
 * $Date: Sun Aug 10 06:42:19 1997$
 *
 Description:
\*____________________________________________________________________________*/
//$SourceTop:$

#include <assert.h>
#include "_PIClass.h"
#include "_PIBase.h"
#include "_PIObj.h"

/*____________________________________________________________________________*\
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int _PIClass::Constructor( _PIObject *pObj, int iArgc,
	const char *ppArgv[] ) const
{
	PIMUSTBETRUE( pObj );
	assert( !pObj->bConstructed );

	/* ---
	Set bConstructed to true ** BEFORE ** calling the
	constructor so that if the constructor polls on a socket
	and an abort occurs, all hell will not break loose
	--- */
	pObj->bConstructed = true;
	int iRet = ( aFunctionFrame[CONSTRUCTOR] )( pObj, iArgc, ppArgv );
	return iRet;
}

/*____________________________________________________________________________*\
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int _PIClass::CopyConstructor( _PIObject *pObj, int iArgc,
	const char *ppArgv[] ) const
{
	PIMUSTBETRUE( pObj );
	assert( !pObj->bConstructed );

	/* ---
	Set bConstructed to true ** BEFORE ** calling the
	constructor so that if the constructor poll on a socket
	and an abort occurs, all hell will not break loose
	--- */
	pObj->bConstructed = true;
	int iRet = ( aFunctionFrame[COPYCONSTRUCTOR] )( pObj, iArgc, ppArgv );
	return iRet;
}

/*____________________________________________________________________________*\
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int _PIClass::Destructor( _PIObject *pObj, int iArgc,
	const char *ppArgv[] ) const
{
	PIMUSTBETRUE( pObj );
	assert( pObj );
	assert( pObj->bConstructed );
	int iRet = ( aFunctionFrame[DESTRUCTOR] )( pObj, iArgc, ppArgv );
	pObj->bConstructed = false;
	return iRet;
}

