/*____________________________________________________________________________*\
 *

 Copyright (c) 1997 John Roy. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    (http://www.johnroy.com/pi3/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: Pi3Expr.cpp$
 * $Date: Sun Aug 10 06:36:33 1997$
 *
 Description:
\*____________________________________________________________________________*/
//$SourceTop:$

#include "Pi3Expr.h"

#include "_Pi3Expr.h"
#include "PIString.h"
#include "PICompat.h"

/*____________________________________________________________________________*\
 *
 Description:
    See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
Pi3Expression *Pi3Expression_new( const char *pExpression,
	FnPi3Write *pParameters, Pi3String *pErrorDescription )
{
	if ( !pExpression || !pErrorDescription )
		{ return 0; };
	/*
	** NOTE:
	** This function (which take void* parameters) is used because of 
	** linkage problems with some microsoft compilers
	*/
	Pi3Expression *pExpr = (Pi3Expression *)_Pi3Expression::Compiler_Bug_Alloc( 
		(void *)pExpression,
		(void *)pParameters,
		(void *)pErrorDescription );
	if ( !pExpr || !pExpr->IsOK() )
		{
		PI_DELETE( pExpr );
		return 0;
		};
	return pExpr;
}

/*____________________________________________________________________________*\
 *
 Description:
    See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
int Pi3Expression_delete( Pi3Expression *pExpr )
{
	if ( !pExpr )
		{ return PIAPI_EINVAL; };
	PI_DELETE( pExpr );
	return PIAPI_COMPLETED;
}

/*____________________________________________________________________________*\
 *
 Description:
    See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
int Pi3Expression_write( Pi3Expression *pExpr,
	PIHTTP *pPIHTTP, void *pData, char *pBuffer, int iBufferLen )
{
	assert( pExpr );
	assert( pPIHTTP );
	if ( !pExpr || !pPIHTTP )
		{ return PIAPI_EINVAL; };
	return pExpr->Write( *pPIHTTP, pData, pBuffer, iBufferLen );
}

