/*____________________________________________________________________________*\
 *

 Copyright (c) 1997 John Roy. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    (http://www.johnroy.com/pi3/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: _PIThread.cpp$
 * $Date: Sun Aug 10 06:29:42 1997$
 *
 Description:
\*____________________________________________________________________________*/
//$SourceTop:$

#include <fstream.h>

#include "OSEntity.h"
#include "PiAPI.h"
#include "Platform.h"
#include "PlatDefs.h"

/*____________________________________________________________________________*\
 *
 Description:
    See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI Thread *PIThread_new( Allocator *pAllocator, int iStackSize )
{
	return Platform::AllocThread( pAllocator, iStackSize );
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIThread_begin( Thread *pThread, ThreadFn fnEntry,
	unsigned long ulData, int iPriority, int iFlags )
{
	if ( !pThread || !fnEntry )
		{ PIERROR( PIAPI_EINVAL ); return PIAPI_ERROR; };
	return pThread->Begin( fnEntry, ulData, iPriority, iFlags ) ?
		PIAPI_COMPLETED : PIAPI_ERROR;
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIThread_terminate( Thread *pThread,
	unsigned long ulExitCode )
{
	if ( !pThread )
		{ PIERROR( PIAPI_EINVAL ); return PIAPI_ERROR; };
	return pThread->Terminate( ulExitCode ) ? PIAPI_COMPLETED : PIAPI_ERROR;
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIThread_delete( Thread *pThread )
{
	if ( !pThread )
		{ PIERROR( PIAPI_EINVAL ); return PIAPI_ERROR; };
	PI_DELETE( pThread );
	return PIAPI_COMPLETED;
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIThread_suspend( Thread *pThread )
{
	if ( !pThread )
		{ PIERROR( PIAPI_EINVAL ); return PIAPI_ERROR; };
	return pThread->Suspend() ? PIAPI_COMPLETED : PIAPI_ERROR;
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIThread_resume( Thread *pThread )
{
	if ( !pThread )
		{ PIERROR( PIAPI_EINVAL ); return PIAPI_ERROR; };
	return pThread->Resume() ? PIAPI_COMPLETED : PIAPI_ERROR;
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI void *PIThread_getSystemHandle( Thread *pThread )
{
	if ( !pThread )
		{ PIERROR( PIAPI_EINVAL ); return 0; };
	PICLEAR;
	return pThread->GetSystemHandle();
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIThread_yield()
{
	Platform::YieldThread();
	return PIAPI_COMPLETED;
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIThread_userYield()
{
	Platform::UserYieldThread();
	return PIAPI_COMPLETED;
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIThread_waitForThread( Thread *pThread )
{
	if ( !pThread )
		{ PIERROR( PIAPI_EINVAL ); return PIAPI_ERROR; };
	return Platform::WaitForThread( pThread )==0 ?
		PIAPI_COMPLETED : PIAPI_ERROR;
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIThread_initThreadKey()
{
	return Platform::AllocThreadKey();
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIThread_destroyThreadKey( int iKey )
{
	return Platform::DeleteThreadKey( iKey ) ?
		PIAPI_COMPLETED : PIAPI_ERROR;
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIThread_setData( Thread *pThread, int iKey, void *pData )
{
	if ( !pThread )
		{ PIERROR( PIAPI_EINVAL ); return PIAPI_ERROR; };
	return pThread->SetData( iKey, pData ) ? PIAPI_COMPLETED : PIAPI_ERROR;
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI void *PIThread_getData( Thread *pThread, int iKey )
{
	if ( !pThread )
		{ PIERROR( PIAPI_EINVAL ); return 0; };
	void *pV = 0;
	PICLEAR;
	if ( !pThread->GetData( iKey, &pV ) )
		{ return 0; };
	return pV;
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI Thread *PIThread_getCurrent()
{
	return Platform::GetCurrentThread();
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI Thread *PIThread_getMain()
{
	return Platform::GetMainThread();
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIThread_getPriority( Thread *pThread, int *piPriority )
{
	if ( !pThread || !piPriority )
		{ PIERROR( PIAPI_EINVAL ); return PIAPI_ERROR; };
	*piPriority = pThread->GetPriority();
	if ( *piPriority == PITHREAD_PRIORITY_INVALID )
		{ return PIAPI_ERROR; };
	return PIAPI_COMPLETED;
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PIThread_setPriority( Thread *pThread, int iNewPriority )
{
	if ( !pThread )
		{ PIERROR( PIAPI_EINVAL ); return PIAPI_ERROR; };
	return pThread->SetPriority( iNewPriority ) ?
		PIAPI_COMPLETED : PIAPI_ERROR;
}

/*____________________________________________________________________________*\
 *
 Description:
	See API header files in ../PIAPI for full description
\*____________________________________________________________________________*/
PUBLIC_PIAPI void PIThread_dbgDump( const char *pDumpFile )
{
	if ( pDumpFile )
		{
		ofstream ofs( pDumpFile, ios::app );
		Platform::ThreadDbgDump( ofs );
		}
	else
		{
		Platform::ThreadDbgDump( cout );
		};
}

