#!/usr/local/bin/perl

#*____________________________________________________________________________*\
#*
#
# Copyright (c) 1997 John Roy. All rights reserved.
#
# These sources, libraries and applications are
# FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
# as long as the following conditions are adhered to.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer. 
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# 3. Redistributions of any form whatsoever and all advertising materials 
#    mentioning features must contain the following
#    acknowledgment:
#    "This product includes software developed by John Roy
#    (http://www.johnroy.com/pi3/)."
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
# GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
#
#*____________________________________________________________________________*|
#*
#* $Source: htmldoc.pl$
#* $Date: Sun Aug 10 06:31:19 1997$
#*
# Description:
#	Create generic HTML documentation from fragements of HTML embedded
#	in source files.
#
# Parameters
#	ARGV[0] - name of HTML file to create. Full path or relative from
#				current directory.
#	ARGV[1] - name of index datafile. Full path or relative from
#				current directory.
#	ARGV[1..n] - Files (relative from current directory, or full path )
#				to look in
#*____________________________________________________________________________*/

#/*___________________________________________________________________________*\
# Global declarations:
#\*___________________________________________________________________________*/
%Hoh;
$current_file = "";

&do_main( @ARGV );

#/*___________________________________________________________________________*\
# *
# Function:
# Synopsis:
# Description:
#\*___________________________________________________________________________*/
sub do_main
	{
	local( $index_file ) = shift;
	local( $data_file ) = shift;
	print "#\n# htmldoc.pl, generating files '$index_file' and '$data_file'\n#\n";
	foreach( @_ )
		{
		print "htmldoc.pl: Scanning $_...\n";
		open( INP, '<'.$_ ) || die "Cannot open file $_ $!\n";
		$current_file = $_;
		&process_file();
		close( INP );
		};
	open( OUT, '>'.$index_file ) ||
		die "Cannot open file $index_file $!\n";
	print OUT '<!--
/*____________________________________________________________________________*\
 *
	*** AUTOMATICALLY GENERATED FILE: DO NOT MODIFY ***
\*____________________________________________________________________________*/
/* $SourceTop:$ */

-->

';
	# print index
	&BeginPage();
	print OUT "<HR><DIR COMPACT>\n";
	print OUT "<A NAME=\"___index\"\n";
	local( $tmp );
	foreach $tmp ( sort( keys( %Hoh ) ) )
		{
		local( $can_name ) = split( /:\s*/, $tmp, 2 );
		local ( $name, $para ) = split( /:\s*/, $Hoh{$tmp}, 2 );
		$_ = $name;
		if ( /Name/ )
			{
			print OUT "<A HREF=\"#$can_name\">$can_name</A><BR>\n";
			};
		};
	&EndPage();
	print OUT "</DIR>\n";
	print OUT "<P>\n";
	
	# print data
	foreach $tmp ( sort( keys( %Hoh ) ) )
		{
		local( $can_name, $entry ) = split( /:\s*/, $tmp, 2 );
		local ( $name, $para ) = split( /:\s*/, $Hoh{$tmp}, 2 );
		&PrintParagraph( $name, $para );
		};
	print OUT "\n";
	close( OUT );

	# 
	# generate the datafile for indexing
	#
	open( OUT, '>>'.$data_file ) ||
		die "Cannot open file $data_file: $!\n";
	$_ = $index_file;
	s/\.\.\/\.\.\/Pi3Docs\//\/pidocs\//;
	$index_file = $_;
	foreach $tmp ( sort( keys( %Hoh ) ) )
		{
		local( $can_name, $entry ) = split( /:\s*/, $tmp, 2 );
		if ( $entry =~ "A" ) 
			{
			print OUT "$can_name: ".&GetURLPath($index_file)."#$can_name\n";
			};
		};
	close( OUT );
	};

#/*___________________________________________________________________________*\
# *
# Function:
# Synopsis:
# Description:
#	Recognise sections of HTML by moving the scanner through the
#	following states.
#\*___________________________________________________________________________*/
sub process_file
	{
	local( $state, $paragraph_name, $paragraph );
	$paragraph_name = "";
	$state = 1;
	while( <INP> )
		{
		if ( $state == 1 )
			{
			if ( /\/\*___\+\+\+HTMLDOC_BEGIN\+\+\+___\*\/.*/ )
				{ $state = 2; $paragraph = ""; next; };
			}
		elsif ( $state == 2 )
			{
			next if ( /\/\*___\+\+\+CNF_BEGIN\+\+\+___\*\/.*/ );
			next if ( /\/\*___\+\+\+CNF_END\+\+\+___\*\/.*/ );
			if ( /^\s*(.*):\W*$/ )
				{
				s/\s*(.*):.*\n/$1/;
				local( $new_paragraph_name ) = $_;
				&emit( $paragraph_name, $paragraph);
				$paragraph_name = $new_paragraph_name;
				$paragraph="";

				next;
				}
			elsif ( /\/\*___\+\+\+HTMLDOC_END\+\+\+___\*\/.*/ )
				{
				&emit( $paragraph_name, $paragraph);
				$paragraph_name = "";
				$paragraph="";

				$state = 1;
				next;
				};
			};
		$paragraph = $paragraph.$_;
		};
	};

#/*___________________________________________________________________________*\
# *
# Function:
# Synopsis:
# Description:
#\*___________________________________________________________________________*/
sub emit
	{
	local( $name, $para ) = @_;

	$_=$name;
	if ( /Name/ )
		{
		$entry = 64;
		$canonical_name = &GetCanonicalName($para);
		};
	$entry = $entry+1;

	if ( length( $canonical_name ) > 0 )
		{
		$Hoh{$canonical_name.": ".pack("c", $entry)} = $name.": ".$para;
		};
	}

#/*___________________________________________________________________________*\
# *
# Function:
# Synopsis:
# Description:
#\*___________________________________________________________________________*/
sub BeginPage
	{
	};

#/*___________________________________________________________________________*\
# *
# Function:
# Synopsis:
# Description:
#\*___________________________________________________________________________*/
sub EndPage
	{
	};

#/*___________________________________________________________________________*\
# *
# Function:
# Synopsis:
# Description:
#\*___________________________________________________________________________*/
sub PrintParagraph
	{
	( $name, $para ) = @_;

	$_ = $name;
	if ( /Name/ )
		{
		print OUT "<HR><A NAME=\"".&GetCanonicalName($para)."\">";
		};
	print OUT "<P><B>".$name.":</B><BR>";

	$_ = $name;
	if ( /See Also/ )
		{
		#
		# strip space characters and make each reference to a link;
		#
		$_ = $para;
		s/\s+/ /g; s/[,|.]//g; s/\(\)//g;
		foreach ( split( / / ) )
			{
			local( $name ) = &GetCanonicalName($_);
			print OUT "<A HREF=\"#$name\">$name</A> ";
			};
		print OUT "<P><A HREF=\"#___index\">Return to index</A>\n";
		}
	else
		{
		print OUT $para;
		};
	}

#/*___________________________________________________________________________*\
# *
# Function:
# Synopsis:
# Description:
#\*___________________________________________________________________________*/
sub GetCanonicalName
	{
	$_ = shift;
	s/^(\s*)(.*)(\s*)$/$2/g;
	return $_;
	};

#/*___________________________________________________________________________*\
# *
# Function:
# Synopsis:
# Description:
#\*___________________________________________________________________________*/
sub GetURLPath
	{
	( $_ ) = shift;
	s/\\\\/\//g;
	s/.*\/+Pi3Docs\/+/\/pidocs\//;
	return $_;
	};


